package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import java.util.Date;
import com.bpcl.lpg.core.enums.LpgPaymentStatusEnum;
import java.util.Calendar;
import java.util.Collections;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.Collection;
import org.apache.commons.collections4.CollectionUtils;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import java.util.List;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import com.bpcl.lpg.core.service.LpgRefillService;
import com.bpcl.lpg.core.payment.billdesk.dao.LPGPaymentInfoDao;
import org.slf4j.Logger;
import com.bpcl.core.model.ProcessPendingRefillCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class ProcessPendingRefillJob extends AbstractJobPerformable<ProcessPendingRefillCronjobModel>
{
    private static final Logger LOG;
    private LPGPaymentInfoDao lpgPaymentInfoDao;
    private LpgRefillService lpgRefillService;
    private NotificationEngineService notificationEngineService;
    private ConfigurationService configurationService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final ProcessPendingRefillCronjobModel cronjobModel) {
        final Long processTimeInMs = cronjobModel.getProcessTimeInMs();
        final Long maxProcessTimeInMs = cronjobModel.getMaxProcessTimeInMs();
        try {
            final List<LpgPaymentInfoModel> processTxns = this.getTransactionForProcessing(processTimeInMs);
            ProcessPendingRefillJob.LOG.info("Found {} refill as [INPROGRESS]", (Object)processTxns.size());
            processTxns.stream().forEach(processTxn -> this.getLpgRefillService().fetchPaymentStatusAndUpdate(processTxn, maxProcessTimeInMs));
            ProcessPendingRefillJob.LOG.info("All the {} refills are processed.", (Object)processTxns.size());
            return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
        }
        catch (final Exception ex) {
            ProcessPendingRefillJob.LOG.error(ex.getMessage(), ex.getCause());
            final String body = MessageFormat.format("Process pending refill cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Process Pending Refill Job Alert", body, this.configurationService.getConfiguration().getString("lpg.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
    }
    
    public boolean isAbortable() {
        return true;
    }
    
    private List<LpgPaymentInfoModel> getTransactionForProcessing(final Long processTimeInMs) {
        final List<LpgPaymentInfoModel> paymentInfos = this.lpgPaymentInfoDao.getRefillInProgressPaymentInfo();
        if (CollectionUtils.isNotEmpty((Collection)paymentInfos)) {
            return paymentInfos.stream().filter(paymentInfo -> this.hasEligibleProcessing(processTimeInMs, paymentInfo)).collect((Collector<? super Object, ?, List<LpgPaymentInfoModel>>)Collectors.toList());
        }
        return Collections.emptyList();
    }
    
    private boolean hasEligibleProcessing(final Long processTimeInMs, final LpgPaymentInfoModel paymentInfo) {
        final Date currentDate = Calendar.getInstance().getTime();
        final long difference = currentDate.getTime() - paymentInfo.getTransactionUTimeStamp().getTime();
        return LpgPaymentStatusEnum.SUCCESSFUL.equals((Object)paymentInfo.getTransactionStatus()) && processTimeInMs < difference;
    }
    
    public void setLpgPaymentInfoDao(final LPGPaymentInfoDao lpgPaymentInfoDao) {
        this.lpgPaymentInfoDao = lpgPaymentInfoDao;
    }
    
    public LpgRefillService getLpgRefillService() {
        return this.lpgRefillService;
    }
    
    public void setLpgRefillService(final LpgRefillService lpgRefillService) {
        this.lpgRefillService = lpgRefillService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(ProcessPendingRefillJob.class);
    }
}
